@@ -1,23 +1,43 @@
-# coding=utf-8
-from __future__ import unicode_literals
 import imgkit
 import numpy as np
 from PIL import Image
 import re
 import io
 import base64
+from lxml import etree
+import xml.etree.ElementTree as ET
+
+
+def is_empty(value):
+    if value:
+        if isinstance(value, str):
+            return len(value.strip()) == 0
+    return True
+
+
+def get_border_height(value):
+    if is_empty(value):
+        return "0"
+    return "2px"
 
 
 def create(head, body, footer, width, height, encode_files=False,
            force_black=True, zoom=1):
     # Add UTF-8 tag and line below head text
     head = '<meta charset="UTF-8"/><div style="height: {height}px; width: ' \
-           '{width}px; position: relative;">{head}<div style="border-left: ' \
-           '{width}px solid grey; height: 2px;"></div>'.format(
-               height=height - 16, width=width - 15, head=head)
+           '{width}px; position: relative;">{head}<div style="display: block; ' \
+           'width: {width}px; height: {border};background: grey;"></div>'.format(
+        height=height - 16,
+        width=width-15, head=head,
+        border=get_border_height(head)
+    )
     # Add line above footer text
-    footer = '<div style="border-left: {width}px solid grey; height: 2px;">'\
-             '</div>{footer}'.format(width=width - 15, footer=footer)
+    footer = '<div style="display: block; width: {width}px; height: {border};background: grey;">' \
+             '</div>{footer}'.format(
+        width=width-15,
+        footer=footer,
+        border=get_border_height(footer)
+    )
     # Create actual footer. This just creates normal invisble footer and actual
     #  footer is put to the bottom with absolute. Otherwise body could overlap
     #  with footer and checking image size doesn't notice that
@@ -41,6 +61,17 @@ def get_lines(html):
         html = html[match.end():]
     return lines
 
+def estimate_size(html, options):
+    image = io.BytesIO(imgkit.from_string(html, False, options=options))
+    im = Image.open(image)
+    return im.size
+
+def estimate_header_size(html, options):
+    tree = etree.fromstring(html, etree.HTMLParser())
+    p_tag = tree.findall('.//p')
+    p_tag[0].find('..').attrib.clear()
+    p_tags_str = ET.tostring(p_tag[0].find('..')).decode()
+    return estimate_size(p_tags_str, options)
 
 def generate_images(head, lines, footer, width, height, encode_files,
                     force_black, zoom):
@@ -50,26 +81,38 @@ def generate_images(head, lines, footer, width, height, encode_files,
     img_count = 0
     images = []
     # Generate as many images as necessary
-    while True:
+    for _ in range(100):
         n = 0
         html = head
-        for line in range(0, len(lines) - skips):
-            html = '{html}{line}'.format(html=html, line=lines[line])
-            n += 1
-        html = '{html}{footer}'.format(html=html, footer=footer)
+
         options = {'width': max_width, 'encoding': 'UTF-8', 'format': 'png',
                    'quiet': '', 'quality': 90}
+
+        header_size = estimate_header_size(head, options)
+        footer_size = estimate_size(footer, options)
+
+        for line in range(0, len(lines) - skips):
+            html = '{html}{line}'.format(html=html, line=lines[line])
+            html = update_svg_size(html, max_width, max_height, header_size, footer_size)
+            n += 1
+        html = '{html}{footer}'.format(html=html, footer=footer)
+
         image = io.BytesIO(imgkit.from_string(html, False, options=options))
         im = Image.open(image)
         width, height = im.size
+
         # Check height
         if height > max_height:
             skips += 1
             # Generate the image, cannot do nothing to fix fitting issue
-            if len(lines) - skips == 0:
-                images.append(generate_image(html, max_width, max_height,
-                                             encode_files, force_black, zoom))
-                return images
+            # Generate the image, don't skip the rest when the height exceeds the max height
+            if height > max_height:
+                skips += 1
+                # Generate the image, cannot do nothing to fix fitting issue
+                if len(lines) - skips <= 0:
+                    images.append(generate_image(html, max_width, max_height,
+                                                 encode_files, force_black, zoom))
+                    return images
         else:
             # Generate the image in correct size
             images.append(generate_image(html, max_width, max_height,
@@ -86,8 +129,35 @@ def generate_images(head, lines, footer, width, height, encode_files,
                 for line in range(0, n):
                     del lines[0]
                 n = 0
+
     return images
 
+def height_on_one_line_exceed_max_height(line_count, skips):
+    """
+    Helper method to identify the height of a line based on skipped lines and
+    the number of lines
+    """
+    return line_count - skips == 0 and line_count > 1
+
+def update_svg_size(html, max_width, max_height, header_size, footer_size):
+    tree = etree.fromstring(html, etree.HTMLParser())
+    p_tag = tree.find('.//p')
+    svgs = tree.findall('.//svg')
+
+    def abs_size(size):
+        return 10 if size <= 0 else size
+
+    if svgs:
+        for svg in svgs:
+            min_max_width_height = min(
+                abs_size(max_width), abs_size(max_height-footer_size[1]-header_size[1])
+            )
+            width_height = min_max_width_height
+            svg.attrib['width'] = str(width_height)
+            svg.attrib['height'] = str(width_height)
+        return ET.tostring(tree).decode()
+    return html
+
 
 def generate_image(html, max_width, max_height, encode_files,
                    force_black=True, zoom=1):
